--[[=============================================================================
    Commands received from the AVSwitch proxy (ReceivedFromProxy)

    Copyright 2024 Key Digital Systems. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_messages = "2015.03.31"
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Power Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: CONNECT_OUTPUT
	Parameters:
		output:			mod 1000 value of Output Connection id
		class:			connection class
		output_id:		value of Output Connection id
--]]
function CONNECT_OUTPUT(output, class, output_id)
	-- Switcher does not support Zone On. Leave cmd blank
	KD_PackAndQueueCommand("")
end

--[[
	Proxy Command: DISCONNECT_OUTPUT
	Parameters:
		output:			mod 1000 value of Output Connection id
		class:			connection class
		output_id:		value of Output Connection id
--]]
function DISCONNECT_OUTPUT(output, class, output_id)
	-- Switcher does not support Zone Off. Leave cmd blank
    KD_PackAndQueueCommand("")
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Input Selection and AV Path Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: SET_INPUT
	Parameters:
		idBinding:			proxybindingid of proxy bound to input connection
		output: 			mod 1000 value of Output Connection ID	
		input: 				mod 1000 value of Input Connection ID
		input_id:			Input Connection ID
		class:				Connection Class
		output_id:			Output Connection id
		bSwitchSeparate		Setting of can_switch_separately capability
		bVideo				indicates a video path selection
		bAudio				indicates an audio path selection
--]]
function SET_INPUT(idBinding, output, input, input_id, class, output_id, bSwitchSeparate, bVideo, bAudio)
    local command
	-- Override 'bSwitchSeparate' flag to true. KD switchers can always switch separately. TODO Fix C4 Bug - Figure out why 'bSwitchSeparate' is null
	bSwitchSeparate = true
    if (bSwitchSeparate) then	--device switches audio and video separatley	   
	   local isAudio = (output_id >= 4000)
	   if (isAudio) then
		  startAudioSelectionTimer(output)		--used in DISCONNECT_OUTPUT
		  gOutputToInputAudioMap[output] = input
		  command = string.format("SPO%04dSA%04d", KD_Helper:ConvertToKDID(output), KD_Helper:ConvertToKDID(input))
	   else
		  startHDMIAudioSelectionTimer(output)	--used in DISCONNECT_OUTPUT
		  local mirrored_output_id = getMirroredOutputID(output_id)
		  local mirrorState = getMirroredOutputState(output_id)
		  if (mirrorState == "NO MIRROR ZONE") or (mirrorState == "AUDIO ZONE WITH MIRRORED VIDEO ZONE") or (mirrorState == "VIDEO ZONE WITH MIRRORED AUDIO ZONE") then
			 gOutputToInputMap[output] = input
			 command = string.format("SPO%04dSI%04d", KD_Helper:ConvertToKDID(output), KD_Helper:ConvertToKDID(input))
		  else
			 LogError("SET_INPUT: ERROR - mirrorState is unknown")
			 return
		  end
		  
	   end
    else
		-- We should never be here. Figure out how to dynamically set "bSwitchSeparate"
		LogError("bSwitchSeparate is false. Video/Audio will be mirrored")
		command = string.format("SPO%04dSI%04d", KD_Helper:ConvertToKDID(output), KD_Helper:ConvertToKDID(input))
    end
	
	-- Send cmd
	KD_PackAndQueueCommand(command)
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Volume Control Functions
--		Proxy control will affect all audio outputs: HDMI and External
--		Audio. The proxy commands will only do all or nothing. No separate
--		controls. All separate controls can be found in the device specific
--		commands send from execute commands section
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: MUTE_OFF
	Parameters:
		output:		mod 1000 value of Output Connection id	
--]]
function MUTE_OFF(output)
	-- Send cmd
	local command = string.format("SPD%04dA%s", KD_Helper:ConvertToKDID(output), tAudioStateMap["DISABLE"])
	KD_PackAndQueueCommand(command)
end

--[[
	Proxy Command: MUTE_ON
	Parameters:
		output:		mod 1000 value of Output Connection id	
--]]
function MUTE_ON(output)
	-- Send cmd
	local command = string.format("SPD%04dA%s", KD_Helper:ConvertToKDID(output), tAudioStateMap["ENABLE"])
	KD_PackAndQueueCommand(command)
end

--[[
	Proxy Command: MUTE_TOGGLE
	Parameters:
		output:		mod 1000 value of Output Connection id	
--]]
function MUTE_TOGGLE(output)
	LogError("MUTE_TOGGLE: ERROR - Not supported")
end

--[[
	Proxy Command: SET_VOLUME_LEVEL
	Parameters:
		output:			mod 1000 value of Output Connection id	
		c4VolumeLevel:	volume level to be set represented in Control4 scale (0-100)
--]]
function SET_VOLUME_LEVEL(output, c4VolumeLevel)
	-- Make sure we don't go past it's max volume
	if (c4VolumeLevel > MAX_DEVICE_LEVEL) then
		c4VolumeLevel = MAX_DEVICE_LEVEL
	end

	-- Send cmd
	local command = string.format("SPD%04dAV%02d", KD_Helper:ConvertToKDID(output), tonumber(c4VolumeLevel))
	KD_PackAndQueueCommand(command)
end

--[[
	Helper Function: SET_VOLUME_LEVEL_DEVICE
	Parameters:
		output:				mod 1000 value of Output Connection id	
		deviceVolumeLevel:	volume level to be set represented in device scale (as sepcified in the device's control protocol)
--]]
function SET_VOLUME_LEVEL_DEVICE(output, deviceVolumeLevel)
	-- Send cmd
	local command = string.format("SPD%04dAV%02d", KD_Helper:ConvertToKDID(output), tonumber(deviceVolumeLevel))
	KD_PackAndQueueCommand(command)
end

--[[
	Proxy Command: PULSE_VOL_DOWN
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function PULSE_VOL_DOWN(output)
	-- Send cmd
	local command = string.format("SPD%04dAV%s", KD_Helper:ConvertToKDID(output), tPulseMap["DOWN"])
	KD_PackAndQueueCommand(command)
end

--[[
	Proxy Command: PULSE_VOL_UP
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function PULSE_VOL_UP(output)
	-- Send cmd
	local command = string.format("SPD%04dAV%s", KD_Helper:ConvertToKDID(output), tPulseMap["UP"])
	KD_PackAndQueueCommand(command)
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Helper Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

--[[
	Helper Function: SEND_COMMAND_FROM_COMMAND_TABLE
	Parameters:
		idBinding: proxy id	
		output: mod 1000 value of Output Connection id
		command_name: name of command to be sent
--]]
function SEND_COMMAND_FROM_COMMAND_TABLE(idBinding, output, command_name)
    local output_for_log = output or "nil"
    LogTrace("SEND_COMMAND_FROM_COMMAND_TABLE(), idBinding=" .. idBinding .. ", output=" .. output_for_log .. ", command_name=" .. command_name)
	
	local command = GetCommandFromCommandTable(idBinding, output, command_name)
	
	if (command == nil) then
		LogTrace("command is nil")
	else
		LogTrace("command = " .. command)
		PackAndQueueCommand(command_name, command)
	end			
end

--[[
	Helper Function: GetCommandFromCommandTable
	Parameters:
		idBinding: proxy id	
		output: mod 1000 value of Output Connection id
		command_name: name of command to be returned
--]]
function GetCommandFromCommandTable(idBinding, command_name)
	LogTrace("GetCommand()")
	local t = {}
	
	if (gControlMethod == "IR") then
		t = CMDS_IR
	else
		t = CMDS
	end	

	if (t[idBinding] ~= nil) then
	   if (t[idBinding][command_name] ~= nil) then
		  return t[idBinding][command_name]
	   end
	end
	
	if (t[command_name] ~= nil) then
		return t[command_name]
	else
		LogWarn('GetCommandFromCommandTable: command not defined - '.. command_name)
		return nil
	end	
	
end


--[[=============================================================================
    Additional KD Functions
===============================================================================]]
--[[
	Gets the system status using STA/STAA
--]]
function GetDeviceStatus()
	local command = GetCommandFromCommandTable(0, "STATUS")
	KD_PackAndQueueCommand(command)
end

--[[
	Helper function to log, pack, and queue commands. Eliminates clutter and repetition
--]]
function KD_PackAndQueueCommand(command)
	local caller = debug.getinfo(2).name
	LogTrace("command = " .. command)
	PackAndQueueCommand(caller, command)
end
